#!/usr/bin/env python
#^#
# OVSServices.py
#
# <description>
#
#
# Copyright (C) 2006, 2007 Oracle.  All rights reserved.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation, version 2.
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
# You should have received a copy of the GNU General Public
# License along with this program; if not, write to the
# Free Software Foundation, Inc., 59 Temple Place - Suite 330,
# Boston, MA 021110-1307, USA.
#$#

import OVSSiteDB
import OVSSiteMonitor
import OVSSiteSysAdmin
import OVSSiteUtility
import OVSSiteVM
import OVSSiteVMConfig
import OVSSiteVMInstall
import OVSSiteXenStore
import OVSSiteHA

import OVSXMonitor      #sys_
import OVSXSysAdmin     #sys_
import OVSXUtility      #utl_ & xen_ [for create/remove disk]
import OVSXXenStore     #xen_
import OVSXXenVM        #xen_
import OVSXXenVMConfig  #xen_
import OVSXXenVMInstall #xen_
import OVSXSysInfo      #get_
import OVSXHA           #ha_
import OVSXOVF          #ovf_

import OVSAsyncTaskCore
import OVSAsyncTaskAux
import OVSAsyncTaskSupport

import OVSV2V
import OVSClone
import OVSVMXParser
import OVSVMDKParser

#----------------------------------------------------------
#special function for test xmlrpc call
def echo(msg):
    return "success:echo=%s" % msg

def sleep(secs):
    import time
    secs = float(secs)
    time.sleep(secs)
    return "success:sleep=%f" % secs

#----------------------------------------------------------

from OVSAsyncTaskCore import init_asyncenv, AsyncProxy
from inspect import ismodule
class OVSServices(object):
    def __init__(self):
        #ovs-agent
        init_asyncenv()
        self.async_proxy = AsyncProxy()

        modules = [ eval(attr) for attr in globals() if ismodule(eval(attr)) ]
        for module in modules:
            for name in dir(module):
                _attr = getattr(module, name)
                if callable(_attr) and hasattr(_attr, "exposed") and _attr.exposed == True:
                    setattr(self, name, _attr)

        self.echo = echo
        self.sleep = sleep

        #xenapi
        import xen.xend.XendAPI as XendAPI
        import OVSXendAuthSessions
        XendAPI.auth_manager = OVSXendAuthSessions.instance
        self.xenapi = XendAPI.XendAPI(XendAPI.AUTH_NONE)

        for meth_name in dir(self.xenapi):
            if not meth_name.startswith("_"):
                meth = getattr(self.xenapi, meth_name)
                if callable(meth) and hasattr(meth, 'api'):
                    setattr(self, getattr(meth, 'api'), meth)

        from xen.xend.XendAPI import XendAPIAsyncProxy
        self.async_xenapi = XendAPIAsyncProxy(self.xenapi)

    def _dispatch(self, method, params):
        from xen.xend.XendAPI import XendAPIAsyncProxy
        if not method.startswith(XendAPIAsyncProxy.method_prefix) \
                and not method.startswith(AsyncProxy.async_method_prefix):
            if hasattr(self, method):
                _method = getattr(self, method)
                rs = _method(*params)
                return rs
            else:
                raise Exception("method: <%s> not supported!" % method)
        elif method.startswith(AsyncProxy.async_method_prefix):
            return self.async_proxy._dispatch(method, params)
        elif method.startswith(XendAPIAsyncProxy.method_prefix):
            return self.async_xenapi._dispatch(method, params)
        else:
            raise Exception("method: <%s> not supported!" % method)

