#!/usr/bin/env python
#script for bugz#7674
AGENT_DIR = "/opt/ovs-agent-2.2"

import sys
sys.path.insert(0, AGENT_DIR)

import time
import re
import datetime
from os.path import exists, join
from commands import getoutput, getstatusoutput

#globals
_vm_dict = {}
_error = None
_option_dict = {}

def collect_data():
    global _vm_dict, _error, _option_dict

    status, output = getstatusoutput("xm list")
    #something goes wrong with xend
    if status != 0: 
        _error = status
        return

    for line in output.splitlines()[1:]:
        parts = line.split()
        _vm_dict[parts[0]] = {"domain_id":  parts[1]}
    del _vm_dict["Domain-0"]

    master = None
    from OVSCDS import CDS
    from OVSServerProxy import ServerProxy
    from OVSServerSelection import _get_agent_version
    cds = CDS("master")
    try:
        agt_url = cds.db_master.get("master_agt_url")
        if agt_url:
            master = ServerProxy(agt_url)
            master.agent_version = _get_agent_version(agt_url)
    finally:
        cds.close()

    for vm_name, attr in _vm_dict.items():
        vm_path = join("/OVS/running_pool", vm_name)

        #check if the vm is under ovs management
        attr["managed"] = False
        if master and master.agent_version >= (2, 2, 0, 62):
            rs = master.get_vm_attr(vm_path, "real_path")
            if rs.startswith("success"):
                if "/OVS" in rs:
                    attr["managed"] = True
        elif exists(join(vm_path, "vm.cfg")):
            attr["managed"] = True
        else:
            output = getoutput("xenstore-ls /local/domain/0/backend/vbd/%s" % attr["domain_id"])
            if "/OVS" in output:
                attr["managed"] = True

        if not attr["managed"]:
            continue

        #uuid
        output = getoutput("xenstore-read /local/domain/%s/vm" % attr["domain_id"])
        uuid = output[4:]
        attr["uuid"] = uuid

        #uptime
        output = getoutput("xm uptime %s" % attr["domain_id"])
        attr["uptime"] = re.split(r"\s+", output.splitlines()[-1], maxsplit=2)[-1]

        #autostart
        if _option_dict.get("outputConfigData"):
            if master and master.agent_version >= (2, 2, 0, 62):
                policy = master.get_vm_policy(vm_path)
                if "vm_auto_start" in policy:
                    attr["autostart"] = "TRUE"
                else:
                    attr["autostart"] = "FALSE"
            else:
                attr["autostart"] = "UNKNOWN"

def format_output(delimiter="|"):
    global _vm_dict, _option_dict

    l = []
    for vm_name, attr in _vm_dict.items():
        if attr["managed"]:
            if not _option_dict.get("outputConfigData"):
                l.append("%s %s%s%s%s" % (vm_name, attr["uuid"], delimiter, attr["uptime"], delimiter))
            else:
                l.append("%s %s%s%s%s%s%s" % (vm_name, attr["uuid"], delimiter, attr["uptime"], delimiter, attr["autostart"], delimiter))
    print "%send" % "".join(l)

def main():
    global _error, _option_dict

    #option parser
    d = {"true": True, "false": False}
    for option in sys.argv[1:]:
        parts = option.split("=")
        if len(parts) == 2:
            if parts[1].lower() in d:
                _option_dict[parts[0]] = d[parts[1].lower()]
            else:
                _option_dict[parts[0]] = parts[1]
        else:
            _option_dict[parts[0]] = True

    collect_data()
    format_output()
    if _error:
        sys.exit(_error >> 8)

if __name__ == "__main__":
    main()

